# -*- coding: utf-8 -*-
'''
Taken from: http://www.djangosnippets.org/snippets/154/

Thanks to zakj.
'''
import json
from django.core.serializers import serialize
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models.query import QuerySet
from django.http import HttpResponse


class HttpJsonResponse(HttpResponse):
    def __init__(self, object):
        if isinstance(object, QuerySet):
            content = serialize('json', object)
        else:
            content = json.dumps(
                object, cls=DjangoJSONEncoder, ensure_ascii=False)
        super(HttpJsonResponse, self).__init__(
            content, content_type='application/json')



# -*- coding: utf-8 -*-
import os
from django.utils.translation import ugettext_lazy as _
from django.db import models
from mediastore.conf import settings
from mediastore.models import Media


class PDF(Media):
    help_text = _('Stores a pdf file and the file size.')

    file = models.FileField(_('pdf file'),
        upload_to=os.path.join(settings.MEDIASTORE_FS_PREFIX, 'pdf')
    )
    file_size = models.PositiveIntegerField(_('file size'), editable=False)

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('PDF')
        verbose_name_plural = _('PDFs')

    def save(self, *args, **kwargs):
        self.file_size = self.file.size
        super(PDF, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
from django.db import models


class QueueItemManager(models.Manager):
    def append(self, video):
        return self.create(related_video=video)

    def get_next(self):
        try:
            return self.get_queryset().select_related().\
                filter(error=False).order_by('uploaded')[0]
        except IndexError:
            return None


class VideoManager(models.Manager):
    pass



# -*- coding: utf-8 -*-
from django.contrib import admin
from mediastore.admin import MediaAdmin
from mediastore.mediatypes.swf.models import SWF


class SWFAdmin(MediaAdmin):
    list_display = ('id', 'name', 'created')


admin.site.register(SWF, SWFAdmin)



# -*- coding: utf-8 -*-
import os
import types


def get_file_extension(filename):
    return os.path.splitext(filename)[1][1:].lower()


def has_file_extension(filename, extensions):
    if not isinstance(extensions, (types.ListType, types.TupleType)):
        extensions = (extensions,)
    fileext = get_file_extension(filename).lower()
    return fileext in [x.lower() for x in extensions]



import os
from django.conf import settings


def media_url(url):
    if hasattr(settings, 'STATIC_URL'):
        base_url = settings.STATIC_URL
    else:
        base_url = settings.MEDIA_URL
    return os.path.join(base_url, 'mediastore', url)



'''
InheritanceQuerySet and InheritanceManager taken from django-model-utils:
https://bitbucket.org/carljm/django-model-utils/src/3b66a0a41a61/model_utils/managers.py
'''
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models.fields.related import OneToOneField
from django.db.models.query import QuerySet


class InheritanceQuerySet(QuerySet):
    def select_subclasses(self, *subclasses):
        if not subclasses:
            subclasses = [rel.var_name for rel in self.model._meta.get_all_related_objects()
                          if isinstance(rel.field, OneToOneField)
                          and issubclass(rel.field.model, self.model)]
        new_qs = self.select_related(*subclasses)
        new_qs.subclasses = subclasses
        return new_qs

    def _clone(self, klass=None, setup=False, **kwargs):
        for name in ['subclasses', '_annotated']:
            if hasattr(self, name):
                kwargs[name] = getattr(self, name)
        return super(InheritanceQuerySet, self)._clone(klass, setup, **kwargs)

    def annotate(self, *args, **kwargs):
        qset = super(InheritanceQuerySet, self).annotate(*args, **kwargs)
        qset._annotated = [a.default_alias for a in args] + kwargs.keys()
        return qset

    def iterator(self):
        iter = super(InheritanceQuerySet, self).iterator()
        if getattr(self, 'subclasses', False):
            for obj in iter:
                sub_obj = []
                for s in self.subclasses:
                    try:
                        sub_obj.append(getattr(obj, s))
                    except ObjectDoesNotExist:
                        pass
                if not sub_obj:
                    sub_obj = [obj]
                sub_obj = sub_obj[0]
                if getattr(self, '_annotated', False):
                    for k in self._annotated:
                        setattr(sub_obj, k, getattr(obj, k))

                yield sub_obj
        else:
            for obj in iter:
                yield obj


class InheritanceManager(models.Manager):
    use_for_related_fields = True

    def get_queryset(self):
        return InheritanceQuerySet(self.model)

    def select_subclasses(self, *subclasses):
        return self.get_queryset().select_subclasses(*subclasses)

    def get_subclass(self, *args, **kwargs):
        return self.get_queryset().select_subclasses().get(*args, **kwargs)



from django import forms
from django.contrib import admin
from mediastore.admin import MediaAdmin
from mediastore.mediatypes.download.models import Download


class DownloadAdmin(MediaAdmin):
    list_display = ('id', 'name', 'file_extension',  'file_size', 'created')
    list_filter = ('file_extension',)


admin.site.register(Download, DownloadAdmin)



from django.template import RequestContext
from django.shortcuts import render_to_response


def add(request):
    is_popup = request.GET.get('_is_popup', 1)
    return render_to_response('admin/mediastore/add_select.html', {
        'is_popup': is_popup,
    }, context_instance=RequestContext(request))



# -*- coding: utf-8 -*-
from django.conf.urls import *


urlpatterns = patterns('mediastore.mediatypes.set.views',
    url(r'^$', 'set_list', name='mediastore-set-list'),
    url(r'^(?P<set_slug>[^/]+)/$', 'set_detail', name='mediastore-set-detail'),
    url(r'^(?P<set_slug>[^/]+)/detail/(?P<slug>[^/]+)$', 'media_detail', name='mediastore-set-media-detail'),
)



# -*- coding: utf-8 -*-
from django.http import Http404
from django.views.generic import DetailView, ListView
from mediastore.mediatypes.set.models import Set
from mediastore.models import Media


class SetListView(ListView):
    queryset = Set.objects.all()
    template_name = 'mediastore/set/set_list.html'


set_list = SetListView.as_view()


class SetDetailView(ListView):
    queryset = Set.objects.all()
    template_name = 'mediastore/set/set_detail.html'

    def get_context_data(self, **kwargs):
        set_slug = self.kwargs['set_slug']
        try:
            kwargs['object'] = self.get_queryset().get(slug=set_slug)
        except Set.DoesNotExist:
            raise Http404
        return super(SetDetailView, self).get_context_data(**kwargs)


set_detail = SetDetailView.as_view()


class MediaDetailView(DetailView):
    queryset = Media.objects.all()
    set_queryset = Set.objects.all()
    template_name = 'mediastore/set/media_detail.html'

    def get_queryset(self):
        set_slug = self.kwargs['set_slug']
        try:
            self.set_object = self.set_queryset.get(slug=set_slug)
        except Set.DoesNotExist:
            raise Http404
        queryset = super(MediaDetailView, self).get_queryset()
        queryset = queryset.filter(sets=self.set_object)
        return queryset

    def get_context_data(self, **kwargs):
        kwargs['set'] = self.set_object
        return super(MediaDetailView, self).get_context_data(**kwargs)


media_detail = MediaDetailView.as_view()



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.contrib import admin
from mediastore.admin import MediaAdmin
from mediastore.mediatypes.video.models import Video, QueueItem


class QueueItemInline(admin.StackedInline):
    model = QueueItem
    extra = 0


class VideoAdmin(MediaAdmin):
    list_display = ('id', 'preview', 'name', 'is_ready', 'errors', 'created')
    fieldsets = (
        MediaAdmin.MAIN_FIELDSET,
        (_('Data'), {
            'fields': ('file', 'thumbnail',)
        })
    )
    list_filter = ('is_ready',)
    inlines = [QueueItemInline]

    def errors(self, obj):
        queryset = obj.queueitem_set.all()
        if queryset.count():
            return queryset.filter(error=True).count() != 0
        return False
    errors.boolean = True

admin.site.register(Video, VideoAdmin)



import os
from django.utils.translation import ugettext_lazy as _
from django.db import models
from mediastore.conf import settings
from mediastore.fields import FileField
from mediastore.models import Media
from mediastore.utils.files import get_file_extension


ALLOWED_EXTENSIONS = getattr(settings, 'MEDIASTORE_AUDIO_ALLOWED_FILEXTENSIONS', None)


class Audio(Media):
    help_text = _('Stores an audio file and stores its file extension.')

    file = FileField(_('audio'),
        allowed_extensions=ALLOWED_EXTENSIONS,
        upload_to=os.path.join(settings.MEDIASTORE_FS_PREFIX, 'audio')
    )
    file_extension = models.CharField(_('file extension'), max_length=12,
        null=True, blank=True, editable=False)

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('audio')
        verbose_name_plural = _('audio')

    def save(self, *args, **kwargs):
        self.file_extension = get_file_extension(self.file.name)
        super(Audio, self).save(*args, **kwargs)




# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from mediastore.models import Media
from mediastore.fields.related import MediaField, MultipleMediaField


class Set(Media):
    preview = MediaField(
        related_name='sets_preview',
        null=True, blank=True)
    media = MultipleMediaField(related_name='sets')

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('set')
        verbose_name_plural = _('sets')

    def __iter__(self):
        return iter(self.media.all().select_subclasses())

    def __len__(self):
        return self.media.count()



# -*- coding: utf-8 -*-
import os
from datetime import datetime
from django.utils.translation import ugettext_lazy as _
from django.db import models
from django.db.models.signals import post_save
from mediastore.mediatypes.video.conf import settings
from mediastore.models import Media
from mediastore.mediatypes.video.managers import QueueItemManager, VideoManager
from mediastore.mediatypes.video import convert


def get_file_extension(file):
    name, ext = os.path.splitext(file.name)
    return ext


class Video(Media):
    help_text = _('Stores a flash video file and a preview thumbnail.')

    file = models.FileField(_('video'),
        upload_to=settings.MEDIASTORE_VIDEO_UPLOAD_DIR)
    thumbnail = models.ImageField(_('video thumbnail'),
        null=True, blank=True,
        upload_to=settings.MEDIASTORE_VIDEO_THUMBNAIL_UPLOAD_DIR)
    generated_thumbnail = models.ImageField(_('generated video thumbnail'),
        null=True, blank=True,
        upload_to=settings.MEDIASTORE_VIDEO_AUTO_THUMBNAIL_UPLOAD_DIR)
    is_ready = models.BooleanField(default=False)

    objects = VideoManager()

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('video')
        verbose_name_plural = _('videos')

    def get_thumbnail(self):
        return self.thumbnail or self.generated_thumbnail or None


class QueueItem(models.Model):
    '''
    A queued video file which should be converted into
    the flv format.
    '''
    related_video = models.ForeignKey(Video)
    uploaded = models.DateTimeField(auto_now_add=True)
    error = models.BooleanField(default=False)
    error_message = models.TextField(default='')

    objects = QueueItemManager()

    class Meta:
        app_label = 'mediastore'

    def __unicode__(self):
        return u"Queue: %s" % unicode(self.related_video)

    def append_error_message(self, backend, exception):
        self.error_message += (
            '%(sep)s\n'
            '%(date)s, convertion failed:\n'
            '%(sep)s\n'
            'stdout:\n\n'
            '%(stdout)s\n'
            '%(sep)s\n'
            'stderr:\n\n'
            '%(stderr)s\n'
            '%(sep)s\n\n' % {
                'date': datetime.now().isoformat(),
                'sep': '=' * 50,
                'stdout': backend.stdout,
                'stderr': backend.stderr,
            })


    def process_video(self, backend):
        self.error = False
        self.needs_processing = True

        path = self.related_video.file.path
        media_root = settings.MEDIA_ROOT
        if not media_root.endswith(os.sep):
            media_root += os.sep
        new_dir = os.path.join(media_root,
            settings.MEDIASTORE_VIDEO_UPLOAD_DIR)
        new_path = os.path.join(new_dir, '%s.flv' % self.related_video.pk)
        # create directories if they don't exist yet
        if not os.path.exists(new_dir):
            os.makedirs(new_dir)
        try:
            backend.convert(path, new_path)
            os.remove(path)
            self.related_video.file = new_path[len(media_root):]
            self.related_video.is_ready = True
            self.related_video.save()
        except backend.ConvertionError, e:
            self.error = True
            self.append_error_message(backend, e)
            self.save()

    def process_thumbnail(self, backend):
        path = self.related_video.file.path
        media_root = settings.MEDIA_ROOT
        if not media_root.endswith(os.sep):
            media_root += os.sep
        try:
            thumb_dir = os.path.join(media_root,
                settings.MEDIASTORE_VIDEO_AUTO_THUMBNAIL_UPLOAD_DIR)
            thumb_path = os.path.join(thumb_dir,
                '%s.jpg' % self.related_video.pk)
            # create directories if they don't exist yet
            if not os.path.exists(thumb_dir):
                os.makedirs(thumb_dir)
            backend.thumbnail(path, thumb_path)
            self.related_video.generated_thumbnail = thumb_path[len(media_root):]
        except backend.ConvertionError, e:
            self.append_error_message(backend, e)
        self.related_video.save()

    def process(self):
        print "process queue item #%s, file: %s ..." % (
            self.pk, self.related_video.file)
        Backend = convert.get_convert_backend()
        backend = Backend()

        self.error_message = ''
        self.process_video(backend)
        if not self.error and self.related_video.file:
            self.process_thumbnail(backend)
        if not self.error:
            self.delete()


def auto_add_to_queue(sender, **kwargs):
    created = kwargs['created']
    instance = kwargs['instance']
    if created:
        QueueItem.objects.append(instance)
post_save.connect(auto_add_to_queue, sender=Video)



import os
from django.utils.translation import ugettext_lazy as _
from django.db import models
from mediastore.conf import settings
from mediastore.models import Media


def get_image_mimetype(file):
    try:
        from PIL import Image
    except ImportError:
        import Image
    closed = file.closed
    try:
        if closed:
            file.open()
        image = Image.open(file)
    except IOError:
        return None
    if closed:
        file.close()
    elif hasattr(file, 'seek'):
        file.seek(0)
    return Image.MIME[image.format]


class Image(Media):
    help_text = _('Stores an image and holds attributes like width, height and mimetype.')

    file = models.ImageField(_('image'),
        width_field='width',
        height_field='height',
        upload_to=os.path.join(settings.MEDIASTORE_FS_PREFIX, 'images')
    )
    width = models.IntegerField(_('width'), editable=False)
    height = models.IntegerField(_('height'), editable=False)
    mimetype = models.CharField(_('mimetype'), max_length=32,
        null=True, blank=True, editable=False)

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('image')
        verbose_name_plural = _('images')

    def save(self, *args, **kwargs):
        self.mimetype = get_image_mimetype(self.file)
        super(Image, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _
from taggit.managers import TaggableManager
from .utils.managers import InheritanceManager, InheritanceQuerySet
from .utils import is_media_instance


_media_type_registry = {}
_content_type_to_model_cache = {}


class MediaQuerySet(InheritanceQuerySet):
    def type(self, obj):
        '''
        Filter queryset by media type. First arguments needs to be either a
        model, a model instance or the model._meta.object_name of a Media
        subclass.
        '''
        media_type = None
        # if obj is a string then let's see what the registry knows about it
        if obj in _media_type_registry:
            obj = _media_type_registry[obj]
        try:
            if issubclass(obj, Media):
                media_type = ContentType.objects.get_for_model(obj)
        except TypeError:
            if isinstance(obj, Media):
                media_type = obj.content_type
        if media_type is None:
            raise ValueError('Cannot determine media type from %r' % obj)
        return self.filter(content_type=media_type)

    def list_by_type(self):
        '''
        Will evaluate the queryset.
        '''
        types = {}
        for obj in self.select_subclasses():
            if obj.media_type not in types:
                types[obj.media_type] = []
            types[obj.media_type].append(obj)
        return types


class MediaManager(InheritanceManager):
    def get_queryset(self):
        return MediaQuerySet(self.model)

    def type(self, obj):
        return self.get_queryset().type(obj)

    def list_by_type(self):
        return self.get_queryset().list_by_type()


class Media(models.Model):
    media_type = None
    help_text = None
    icon = None

    # details
    name = models.CharField(max_length=120, null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    slug = models.SlugField(unique=True, blank=True, help_text=_(
        u'Leave this field blank for autopopulating a unique tag '
        u'based on the objects name.'
    ))

    # categorization
    tags = TaggableManager(blank=True)
    content_type = models.ForeignKey(ContentType, editable=False)

    created = models.DateTimeField(auto_now_add=True)

    # managers
    objects = MediaManager()

    class Meta:
        app_label = 'mediastore'
        verbose_name = _('media')
        verbose_name_plural = _('media')
        ordering = ('created',)

    def __repr__(self):
        name = 'Media'
        if self.__class__ is not Media:
            name = 'Media:%s' % self.__class__.__name__
        return '<%s: pk=%d "%s">' % (name, self.pk, self.name or '')

    def __unicode__(self):
        return self.name or u''

    def get_media_type(self):
        if self.media_type:
            return self.media_type
        return self.content_type.model_class().media_type

    def generate_slug(self):
        from mediastore.utils import unique_slug
        return unique_slug(self.name or self.pk, Media)

    def save(self, *args, **kwargs):
        if self.content_type_id is None:
            media_content_type = ContentType.objects.get_for_model(Media)
            content_type = ContentType.objects.get_for_model(self)
            if content_type != media_content_type:
                self.content_type = content_type
        if not self.content_type:
            raise AssertionError('''This media instance has no associated media type. Access Media only by subclasses.''')
        if not self.slug:
            self.slug = self.generate_slug()
        super(Media, self).save(*args, **kwargs)

    @property
    def object(self):
        '''
        Returns the related media item.
        '''
        if not hasattr(self, '_related_media_instance'):
            self._related_media_instance = None
            if is_media_instance(self):
                self._related_media_instance = self
            else:
                model = self.content_type.model_class()
                o2o_rel = model._meta.get_field_by_name('media_ptr')[0]
                attname = o2o_rel.related_query_name()
                self._related_media_instance = getattr(self, attname)
        return self._related_media_instance


def assign_media_types(sender, **kwargs):
    if issubclass(sender, Media):
        if getattr(sender, 'media_type', None) is None:
            sender.media_type = sender._meta.object_name.lower()
        if sender.media_type in _media_type_registry:
            raise AssertionError(
                u'There is already a registered media type "%s"' % sender.media_type)
        _media_type_registry[sender.media_type] = sender

signals.class_prepared.connect(
    assign_media_types,
    dispatch_uid='mediastore.models.assign_media_types')



# -*- coding: utf-8 -*-
import os
import subprocess
from datetime import datetime
from mediastore.mediatypes.video.conf import settings


class VideoConverterBase(object):
    convert_bin = None
    convert_args = None
    thumbnail_bin = None
    thumbnail_args = None

    class ConvertionError(Exception):
        pass

    def __init__(self, **kwargs):
        self.stdout = ''
        self.stderr = ''
        self.failed = False
        self.config = self.prepare_config(kwargs)

    def prepare_config(self, config):
        default_config = {
            'width': settings.MEDIASTORE_VIDEO_WIDTH,
            'height': settings.MEDIASTORE_VIDEO_HEIGHT,
            'bitrate': settings.MEDIASTORE_VIDEO_BITRATE,
            'audio_bitrate': settings.MEDIASTORE_VIDEO_AUDIO_BITRATE,
            'thumbnail_pos': settings.MEDIASTORE_VIDEO_THUMBNAIL_POS,
        }
        default_config.update(config)
        return default_config

    def prepare_command_args(self, bin, args, extra_config=None):
        config = self.config.copy()
        if extra_config:
            config.update(extra_config)
        return [arg % config for arg in [bin] + list(args)]

    def execute_command(self, args):
        print ' '.join(args)
        try:
            self.stdout += ('Executing: "%s", %s\n' % (
                ' '.join(args), datetime.now()))
            try:
                process = subprocess.Popen(
                    args,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                )
                process.wait()
            except OSError:
                raise self.ConvertionError
            self.stdout += process.stdout.read() + '\n'
            self.stderr += process.stderr.read() + '\n'
            self.stdout += 'Exit code: %d\n' % process.returncode
        except self.ConvertionError, e:
            self.stderr += 'Convertion failed:\n%s\n' % e.args[0]
            self.failed = True
            raise

    def convert(self, input_path, output_path, extra_config=None):
        if extra_config is None:
            extra_config = {}
        extra_config['input'] = input_path
        extra_config['output'] = output_path
        self.execute_command(self.prepare_command_args(
            self.convert_bin,
            self.convert_args,
            extra_config))
        # no output file created
        if not os.path.exists(output_path):
            self.stderr += 'convertion: no output file created\n'
            raise self.ConvertionError
        # filesize is 0 bytes
        if os.stat(output_path)[6] == 0:
            os.unlink(output_path)
            self.stderr += 'convertion: zero size output file\n'
            raise self.ConvertionError

    def thumbnail(self, input_path, output_path, extra_config=None):
        if extra_config is None:
            extra_config = {}
        extra_config['input'] = input_path
        extra_config['output'] = output_path
        try:
            self.execute_command(self.prepare_command_args(
                self.thumbnail_bin,
                self.thumbnail_args,
                extra_config))
        except self.ConvertionError:
            try:
                extra_config['thumbnail_pos'] = 1
                self.execute_command(self.prepare_command_args(
                    self.thumbnail_bin,
                    self.thumbnail_args,
                    extra_config))
            except self.ConvertionError:
                pass
        # filesize is 0 bytes
        if os.stat(output_path)[6] == 0:
            os.unlink(output_path)
            self.stderr += 'thumbnail creation: zero size output file\n'
            raise self.ConvertionError
        # no output file created
        if not os.path.exists(output_path):
            self.stderr += 'thumbnail creation: no output file created\n'
            raise self.ConvertionError



# -*- coding: utf-8 -*-
import os
from django import template
from django.template.defaultfilters import stringfilter
from django_templatequery.template import get_models, filter_models
from mediastore.models import Media
from mediastore.preview import display_media, preview_media
from mediastore.utils import get_media_types

register = template.Library()


def _type(queryset, value):
    return queryset.type(value)

option_callbacks = {
    'type': _type,
}


register.tag('get_media', get_models(Media.objects.all(), option_callbacks))
register.tag('filter_media', filter_models(Media, option_callbacks))


def get_display_media_node(display_func, base_class=None):
    if base_class is None:
        base_class = template.Node
    class DisplayMediaNode(base_class):
        def __init__(self, varname, options, limit_type=None):
            self.limit_type = limit_type
            self.var = template.Variable(varname)
            self.options = options

        def render(self, context):
            obj = self.var.resolve(context)
            return display_func(
                obj,
                context,
                extra_context=self.options,
                limit_type=self.limit_type)
    return DisplayMediaNode


def display_media_tag(node, mediatype=None):
    def display_media(parser, token):
        '''
        {% display_media object size=160px %}
        '''
        tokens = token.split_contents()
        tag_name = tokens[0]
        if len(tokens) < 2:
            raise template.TemplateSyntaxError(u'%r expects at least one arguments.' % tag_name)
        varname = tokens[1]
        options = {}
        if len(tokens) > 2:
            for key, value in [x.split('=') for x in tokens[2].split(',')]:
                options[key] = value
        return node(varname, options, limit_type=mediatype)
    return display_media


class MediaTypeRequirements(template.Node):
    template_name = 'mediastore/types/%s/requirements.html'

    def __init__(self, media_types):
        self.media_types = media_types

    def render(self, context):
        requirements = ''
        for media_type in self.media_types:
            context.push()
            try:
                requirements += template.loader.render_to_string(
                    self.template_name % media_type,
                    context
                )
            except template.TemplateDoesNotExist:
                pass
            context.pop()
        return requirements

    @classmethod
    def tag(cls, parser, token):
        '''
        {% media_requirements swf video set %}
        '''
        tokens = token.split_contents()
        tag_name = tokens[0]
        if len(tokens) < 2:
            raise template.TemplateSyntaxError(u'%r expects at least one arguments.' % tag_name)
        return cls(tokens[1:])


class ListMediaByType(template.Node):
    def __init__(self, queryset, as_var):
        self.queryset = template.Variable(queryset)
        self.as_var = as_var

    def render(self, context):
        queryset = self.queryset.resolve(context)
        if not issubclass(queryset.model, Media) and queryset.model is not Media:
            raise ValueError('You must provide a media queryset.')
        context[self.as_var] = queryset.list_by_type()
        return ''

    @classmethod
    def tag(cls, parser, token):
        '''
        {% list_media_by_type media_queryset as dictionary_of_types %}
        '''
        tokens = token.split_contents()
        tag_name = tokens.pop(0)
        if len(tokens) != 3:
            raise template.TemplateSyntaxError(
                '%s needs exactly three arguments.' % tag_name)
        if tokens[1] != 'as':
            raise template.TemplateSyntaxError(
                '%s\'s second argument must be "as".' % tag_name)
        return cls(tokens[0], tokens[2])


register.tag('display_media', display_media_tag(
    get_display_media_node(display_media)))
register.tag('preview_media', display_media_tag(
    get_display_media_node(preview_media)))

# add {% display_media %} tag for every available mediatype
for mediatype in get_media_types():
    register.tag('display_%s' % mediatype,
        display_media_tag(get_display_media_node(display_media), mediatype))
    register.tag('preview_%s' % mediatype,
        display_media_tag(get_display_media_node(preview_media), mediatype))

register.tag('media_requirements', MediaTypeRequirements.tag)
register.tag('list_media_by_type', ListMediaByType.tag)


@register.filter
@stringfilter
def remove_fileextension(value):
    return os.path.splitext(value)[0]


@register.filter
@stringfilter
def get_basename(value):
    return os.path.basename(value)



# -*- coding: utf-8 -*-
from django import template
from mediastore.admin import admin_preview_media
from mediastore.templatetags.mediastore_tags import get_display_media_node, display_media_tag
from mediastore.utils import get_media_types

register = template.Library()


class GetMediaTypesNode(template.Node):
    def __init__(self, as_var, media_types):
        self.as_var = as_var
        self.media_types = media_types

    def render(self, context):
        context[self.as_var] = self.media_types
        return ''


def get_media_types_tag(parser, token):
    '''
    {% get_media_types as mediatypes %}
    '''
    try:
        tag_name, as_, as_var = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(u'%r expects two arguments.' % tag_name)
    media_types = sorted(get_media_types())
    return GetMediaTypesNode(as_var, media_types)
register.tag('get_media_types', get_media_types_tag)


register.tag('admin_preview_media', display_media_tag(
    get_display_media_node(admin_preview_media)))



# -*- coding: utf-8 -*-
import types
from django.db.models.query import QuerySet
from django import template
from .models import Media
from .utils import get_content_types_for_type


def _get_template_names(templates, mediatype):
    # ``templates`` is a single template string
    if isinstance(templates, basestring):
        templates = (templates,)
    templates = list(templates)
    for i, template_name in enumerate(templates):
        try:
            templates[i] = template_name % {
                'mediatype': mediatype,
            }
        except (KeyError, TypeError):
            pass
    return templates


def filter_media(media, limit_type=None):
    if limit_type is None:
        return media
    if hasattr(media, '__iter__'):
        if isinstance(media, QuerySet):
            cts = get_content_types_for_type(limit_type)
            return media.filter(content_type__in=cts)
        return list(filter(lambda o: o.get_media_type() == limit_type, media))


def display_media(obj, context=None, extra_context=None,
        template_name_single=None, template_name_many=None,
        limit_type=None):
    if context is None:
        context = template.Context()
    if isinstance(context, types.DictType):
        context = template.Context(context)
    context.push()
    if extra_context:
        context.update(extra_context)

    template_name_single = template_name_single or 'mediastore/types/%(mediatype)s/display.html'
    template_name_many = template_name_many or 'mediastore/types/%(mediatype)s/display_many.html'

    result = u''

    if isinstance(obj, (QuerySet, types.ListType, types.TupleType)):
        many = True
        obj = filter_media(obj, limit_type=limit_type)
        object_list = [x.object for x in obj]
        if len(object_list) == 0:
            context.pop()
            return u''
        media_type = object_list[0].get_media_type()
        for obj in object_list:
            if obj.get_media_type() != media_type:
                raise ValueError(u'Only media objects with the same type are allowed.')
        context['object_list'] = object_list
        try:
            result = template.loader.render_to_string(
                _get_template_names(template_name_many, media_type)[0],
                context,
            )
        except template.TemplateDoesNotExist:
            result = u''
            for obj in context['object_list']:
                context.push()
                context['object'] = obj
                result += template.loader.render_to_string(
                    _get_template_names(template_name_single, media_type)[0],
                    context,
                )
                context.pop()
    elif isinstance(obj, Media):
        many = False
        context['object'] = obj.object
        media_type = obj.get_media_type()
        result = template.loader.render_to_string(
            _get_template_names(template_name_single, media_type)[0],
            context,
        )

    if extra_context:
        context.pop()
    context.pop()
    return result


def preview_media(*args, **kwargs):
    kwargs['template_name_single'] = 'mediastore/types/%(mediatype)s/preview.html'
    kwargs['template_name_many'] = 'mediastore/types/%(mediatype)s/preview_many.html'
    return display_media(*args, **kwargs)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.db import models
from django import forms
from mediastore.utils.files import has_file_extension


class FileFormField(forms.FileField):
    default_error_messages = {
        'invalid': _(u"No file was submitted. Check the encoding type on the form."),
        'missing': _(u"No file was submitted."),
        'empty': _(u"The submitted file is empty."),
        'max_length': _(u'Ensure this filename has at most %(max)d characters (it has %(length)d).'),
        'wrong_extension': _(u'Ensure that you only upload %(extensions)s files.'),
        'validation_failed': _(u'Ensure that you upload a valid file.'),
    }

    def __init__(self, *args, **kwargs):
        self.allowed_extensions = kwargs.pop('allowed_extensions', None)
        self.validation_function = kwargs.pop('validation_function', None)
        super(FileFormField, self).__init__(*args, **kwargs)

    def clean(self, data, initial=None):
        data = super(FileFormField, self).clean(data, initial)
        if self.allowed_extensions is not None:
            if not has_file_extension(data.name, self.allowed_extensions):
                raise forms.ValidationError(
                    self.error_messages['wrong_extension'] % {
                        'extensions': ', '.join(self.allowed_extensions)
                    })
        if self.validation_function:
            if not self.validation_function(data):
                raise forms.ValidationError(
                    self.error_messages['validation_failed'])
        return data


class FileField(models.FileField):
    def __init__(self, *args, **kwargs):
        self.allowed_extensions = kwargs.pop('allowed_extensions', None)
        self.validation_function = kwargs.pop('validation_function', None)
        super(FileField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': FileFormField,
            'allowed_extensions': self.allowed_extensions,
            'validation_function': self.validation_function,
        }
        defaults.update(kwargs)
        return super(FileField, self).formfield(**defaults)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages


class UltraMagicString(object):
    '''
    Taken from
    http://stackoverflow.com/questions/1162338/whats-the-right-way-to-use-unicode-metadata-in-setup-py
    '''

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value

    def __unicode__(self):
        return self.value.decode('UTF-8')

    def __add__(self, other):
        return UltraMagicString(self.value + str(other))

    def split(self, *args, **kw):
        return self.value.split(*args, **kw)


long_description = UltraMagicString(u'\n\n'.join((
    file('README.rst').read(),
    file('CHANGES.rst').read(),
)))

packages = find_packages('.')
packages = [p for p in packages if p.startswith('mediastore.')]
packages = ['mediastore'] + packages

setup(
    name = 'django-mediastore',
    version = '0.7.1',
    url = 'https://bitbucket.org/gregmuellegger/django-mediastore',
    license = 'BSD',
    description = '',
    long_description = long_description,
    author = UltraMagicString('Gregor Müllegger'),
    author_email = 'gregor@muellegger.de',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ),
    packages = packages,
    include_package_data = True,
    zip_safe = False,
    install_requires = [
        'setuptools',
        'django-sortedm2m',
        'django-taggit',
        'django-templatequery',
    ],
    tests_require = [],
    test_suite = 'runtests.runtests',
)
